/*********************** adjust_user.c ***************************/

/* Assuming a user has just been created on Digital UNIX with C2 security
   turned on, this utility will set the user up with an "invalid" password
   and set the password validity time to 50 years. */

/******************************************************************************
 * Copyright (c) 1995,1996 Donated to the public domain                       *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

#include <sys/types.h>
#include <sys/security.h>
#include <prot.h>
#include <time.h>

int main( argc, argv )
int    argc;
char **argv;
{
   int                eternity1, eternity2;
   struct pr_passwd  *entry_ptr, entry_buf;

   set_auth_parameters( argc, argv );

   eternity1 = 3600 * 24;       /* 1 day */
   eternity1 = eternity1 * 366; /* 1 year */
   eternity2 = eternity1;
   eternity1 = eternity1 * 50;  /* 50 years */
   eternity2 = eternity2 * 51;  /* 51 years */

   if (argc != 2)
   {
      printf("%s: needs exactly one argument, username\n", argv[0] );
      return 1;
   }

   entry_ptr = getprpwnam( argv[1] );
   if (entry_ptr == NULL)
   {
      printf("%s: username \"%s\" does not exist\n", argv[0], argv[1] );
      return 1;
   }

   entry_buf = *entry_ptr;

   entry_buf.ufld.fd_encrypt[0] = '*';
   entry_buf.ufld.fd_encrypt[1] = '\0';
   entry_buf.uflg.fg_encrypt    = 1;

   entry_buf.ufld.fd_schange    = time( (time_t *) 0 );
   entry_buf.uflg.fg_schange    = 1;

   entry_buf.ufld.fd_pwchanger  = 0;   /* changed by user "root"=0 */
   entry_buf.uflg.fg_pwchanger  = 1;

   entry_buf.ufld.fd_min        = eternity1; /* a long time before password */
   entry_buf.uflg.fg_min        = 1;         /* change is required */

   entry_buf.ufld.fd_expire     = eternity2; /* a long time before password */
   entry_buf.uflg.fg_expire     = 1;         /* expiry */

   entry_buf.ufld.fd_lifetime   = eternity2; /* a long time before account  */
   entry_buf.uflg.fg_lifetime   = 1;         /* expiry */

   if (!putprpwnam( argv[1], &entry_buf ))
   {
      printf("%s: cannot update database entry for user \"%s\"\n",
              argv[0], argv[1] );
      return 1;
   }

   endprpwent();
   return 0;
}
