/********************** down_d.c **********************/

/* Invoke a command with reduced privilege.
   Reduced privilege means:  a specific user id (obtained from a username)
                             a specific group id (obtained from a grpname)
                             a new root directory */

/******************************************************************************
 * Copyright (c) 1995,1996 Donated to the public domain                       *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

#include <syslog.h>
#include <stdio.h>
#include <string.h>
#include "drop_priv.h"

#ifdef USE_SYSLOG
#ifdef USE_PRTF
   I need either USE_SYSLOG or USE_PRTF defined, not both
#else
#endif
#else
#ifdef USE_PRTF
#else
   I need either USE_SYSLOG or USE_PRTF defined
#endif
#endif

void syntax( prgname )
char *prgname;
{
#ifdef USE_SYSLOG
      syslog( LOG_ERR,
         "Syntax: %s username:grpname command parm1 parm2 ...\n", prgname );
      syslog( LOG_ERR, "        or\n" );
      syslog( LOG_ERR,
        "        %s username:grpname:dirname command parm1 parm2 ... \n",
        prgname );
#else
      fprintf( stderr,
         "Syntax: %s username:grpname command parm1 parm2 ...\n", prgname );
      fprintf( stderr, "        or\n" );
      fprintf( stderr,
        "        %s username:grpname:dirname command parm1 parm2 ... \n",
        prgname );
#endif
      exit(1);
}

int main( argc, argv )
int argc;
char **argv;
{
   char *my_username = NULL;
   char *my_grpname  = NULL;
   char *my_dirname  = NULL;
   char *tmp_ptr     = NULL;

   if (argc < 3)
   {
      syntax( argv[0] ); return 1;
   }

   tmp_ptr = strchr( argv[1], ':' );

   if ((tmp_ptr == NULL) || (tmp_ptr == argv[1]))
   {
      syntax( argv[0] ); return 1;
   }

   my_username  = argv[1];
   *(tmp_ptr++) = '\0';
   my_grpname   =  tmp_ptr;

   if (*(my_grpname) == '\0')
   {
      syntax( argv[0] ); return 1;
   }

   tmp_ptr      = strchr( my_grpname, ':' );

   if (tmp_ptr != NULL)
   {
      if (tmp_ptr == my_grpname)
      {
         syntax( argv[0] ); return 1;
      }

      *(tmp_ptr++) = '\0';
      my_dirname   = tmp_ptr;

      if (*(my_dirname) == '\0')
      {
         syntax( argv[0] ); return 1;
      }
   }

#ifdef 0
#ifdef USE_SYSLOG
      syslog( LOG_INFO, "Before drop_priv" );
#else
      fprintf( stderr, "Before drop_priv\n" );
#endif
#endif

   if (!drop_priv( my_username, my_grpname, my_dirname ))
   {
#ifdef USE_SYSLOG
      syslog( LOG_ERR, "Cannot drop privileges\n" );
#else
      fprintf( stderr, "Cannot drop privileges\n" );
#endif
      return 1;
   }

#ifdef 0
#ifdef USE_SYSLOG
      syslog( LOG_INFO, "After drop_priv" );
#else
      fprintf( stderr, "After drop_priv\n" );
#endif
#endif

   {
      char **arg_buf = NULL;
      int    our_idx = 0;

      arg_buf = (char **) malloc( sizeof(char *) * (argc - 1) );
      if (arg_buf == NULL)
      {
#ifdef USE_SYSLOG
         syslog( LOG_ERR, "Cannot allocate parameter area\n" );
#else
         fprintf( stderr, "Cannot allocate parameter area\n" );
#endif
         return 1;
      }

      for (our_idx = 2; our_idx < argc; our_idx++)
         arg_buf[ our_idx - 2 ] = argv[ our_idx ];
      arg_buf[ argc - 2 ] = NULL;
      (void) execv( argv[2], arg_buf );
   }
   return 1;
}
