/********************** genmailpasswdfile.c ***********************/

/******************************************************************************
 * Copyright (c) 1996 Donated to the public domain                            *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

/* This program is so silly I feel bad about it. Takes four parameters:

                - the name of a directory
                - the name of three UNIX user names

   The program generates a file called "passwd" which is created in the
   directory passed as a parameter. It will then read the real password
   file and pick up the username entries matching the three UNIX user names
   passed as parameters, plus all entries for which the user name has the
   group "system" as its principle group. Any other username entries
   are ignored.

   For the username entries that are NOT ignored, this program creates
   username entries in the outputted file "passwd". Only the username,
   userid, groupid, and GECOS (account description) fields are copied to
   the new file. The outputted password field is fixed at "*", the
   outputted home directory field is fixed at "/", and the outputted
   shell is fixed at "".

   This program was specifically built for the chrooted sendmail setup
   that is a part of S4.

   I doubt this program can be reused for anything else in a useful
   manner. If somebody actually wants this garbage code it is free
   for the taking to everybody...

   I believe this program may be useful in a Comp.Sci. undergraduate
   course to show new students the kind of programs they should NOT write.

           Marc Chatel, Feb. 1996 */



#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>


void usage( prg, msg )
char *prg, *msg;
{
   fprintf( stderr, "%s: error, %s\n", prg, msg );
}

int main( argc, argv )
int argc;
char **argv;
{
   char            filename_buf[BUFSIZ];
   FILE           *new_pwdfile;
   int             rc;
   struct passwd  *pwd_ptr;
   struct group   *grp_ptr;
   gid_t           system_group;

   new_pwdfile = NULL; pwd_ptr = NULL; grp_ptr = NULL;

   if (argc < 5)
   {
      usage( argv[0], "bad parameters" ); return 1;
   }

   /* argv[1] should be the directory where the simplified file "passwd"
      is created */

   if (strlen( argv[1] ) >= BUFSIZ - 20)
   {
      usage( argv[0], "directory parameter too long" ); return 1;
   }

   strcpy( filename_buf, argv[1] );
   strcat( filename_buf, "/passwd" );

   new_pwdfile = fopen( filename_buf, "w" );
   if (new_pwdfile == NULL)
   {
      fprintf( stderr, "Cannot open file \"%s\" for writes\n", filename_buf );
      return 1;
   }

   grp_ptr = getgrnam( "system" );

   if (grp_ptr == NULL)
   {
      fprintf( stderr, "Cannot read entry for group \"system\"\n" );
      return 1;
   }

   system_group = grp_ptr->gr_gid;
   
   rc = setpwent();

   /* Gather info for users "root", the sendmail user name (argv[2]),
      the pop user name (argv[3]), the popadmin user name (argv[4]),
      and all users in group "system" */

   while ((pwd_ptr = getpwent()) != NULL)
   {
      if ((strcmp( pwd_ptr->pw_name, "root" )  == 0)  ||
          (strcmp( pwd_ptr->pw_name, argv[2] ) == 0)  ||
          (strcmp( pwd_ptr->pw_name, argv[3] ) == 0)  ||
          (strcmp( pwd_ptr->pw_name, argv[4] ) == 0)  ||
          (pwd_ptr->pw_gid == system_group))
      {
         fprintf( new_pwdfile, "%s:*:%ld:%ld:%s:/:\n",
                  pwd_ptr->pw_name, (long) pwd_ptr->pw_uid,
                  (long) pwd_ptr->pw_gid, pwd_ptr->pw_gecos  );
      }
   }
   endpwent();

   fclose( new_pwdfile );
}
